/* FILE: myanisotropy.cc            -*-Mode: c++-*-
 *
 * Simple uniaxial anisotropy, derived from Oxs_Energy class.
 *
 */

#include <string>

#include "oc.h"
#include "nb.h"
#include "threevector.h"
#include "director.h"
#include "simstate.h"
#include "ext.h"
#include "key.h"
#include "mesh.h"
#include "meshvalue.h"
#include "myanisotropy.h"
#include "energy.h"		// Needed to make MSVC++ 5 happy

OC_USE_STRING;

// Oxs_Ext registration support
OXS_EXT_REGISTER(My_Anisotropy);

/* End includes */


// Constructor
My_Anisotropy::My_Anisotropy(
  const char* name,     // Child instance id
  Oxs_Director* newdtr, // App director
  const char* argstr)   // MIF input block parameters
  : Oxs_Energy(name,newdtr,argstr), mesh_id(0)
{
  // Process arguments
  OXS_GET_INIT_EXT_OBJECT("K1",Oxs_ScalarField,K1_init);
  OXS_GET_INIT_EXT_OBJECT("K2",Oxs_ScalarField,K2_init);
  OXS_GET_INIT_EXT_OBJECT("axis",Oxs_VectorField,axis_init);
  VerifyAllInitArgsUsed();
}

OC_BOOL My_Anisotropy::Init()
{
  mesh_id = 0;
  K1.Release();
  K2.Release();
  axis.Release();
  return Oxs_Energy::Init();
}

void My_Anisotropy::GetEnergy
(const Oxs_SimState& state,
 Oxs_EnergyData& oed
 ) const
{
  const Oxs_MeshValue<OC_REAL8m>& Ms_inverse = *(state.Ms_inverse);
  const Oxs_MeshValue<ThreeVector>& spin = state.spin;

  // Use supplied buffer space, and reflect that use in oed.
  oed.energy = oed.energy_buffer;
  oed.field = oed.field_buffer;
  Oxs_MeshValue<OC_REAL8m>& energy = *oed.energy_buffer;
  Oxs_MeshValue<ThreeVector>& field = *oed.field_buffer;

  OC_INDEX size = state.mesh->Size();
  if(mesh_id != state.mesh->Id()) {
    // This is either the first pass through, or else mesh
    // has changed.
    mesh_id=0;
    K1_init->FillMeshValue(state.mesh,K1);
    K2_init->FillMeshValue(state.mesh,K2);
    axis_init->FillMeshValue(state.mesh,axis);
    mesh_id=state.mesh->Id();
  }

  for(OC_INDEX i=0;i<size;++i) {
    OC_REAL8m field_mult = Ms_inverse[i]/MU0;
    if(field_mult==0.0) {
      energy[i]=0.0;
      field[i].Set(0.,0.,0.);
      continue;
    }

    const ThreeVector& u1 = axis[i];
    const ThreeVector&  m = spin[i];
    const OC_REAL8m k1 = K1[i];
    const OC_REAL8m k2 = K2[i];

    OC_REAL8m dot = m*u1;
#if 0
    energy[i] = -k1*dot*dot;
    field[i] = 2*k1*dot*field_mult*u1;
#else
    OC_REAL8m dotsq = dot*dot;
    energy[i] = ((dotsq-2)*k2-k1)*dotsq;
    field[i] = (2*field_mult*(k1+2*(1-dotsq)*k2)*dot)*u1;
#endif
  }
}
